#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test WorkflowModules."""
# pylint: disable=too-many-public-methods, protected-access, too-many-lines

from __future__ import print_function
from __future__ import absolute_import
import unittest
import copy
import os
import shutil
import tempfile
import sys
try:
  from StringIO import StringIO  # for Python 2
except ImportError:
  from io import StringIO  # for Python 3
from mock import MagicMock as Mock, patch

from DIRAC import gLogger, S_ERROR, S_OK
from DIRAC.RequestManagementSystem.Client.Request import Request
from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.ProductionData import getLogPath
from ILCDIRAC.Workflow.Modules.FailoverRequest import FailoverRequest
from ILCDIRAC.Workflow.Modules.UploadOutputData import UploadOutputData
from ILCDIRAC.Workflow.Modules.UploadLogFile import UploadLogFile
from ILCDIRAC.Workflow.Modules.UserJobFinalization import UserJobFinalization

from Tests.Utilities.OperationsMock import createOperationsMock
from six.moves import range

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules'

#from DIRAC.Workflow.Modules.test.Test_Modules import ModulesTestCase as DiracModulesTestCase
#import DIRAC.Workflow.Modules.test.Test_Modules as Test_Modules
gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)


def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass


class ModulesTestCase (unittest.TestCase):
  """ILCDirac version of Workflow module tests."""

  @patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
  @patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
  @patch("%s.ModuleBase.Operations" % MODULE_NAME, new=Mock())
  @patch("%s.UploadLogFile.StorageElement" % MODULE_NAME, new=Mock(return_value=S_OK()))
  @patch("DIRAC.Resources.Storage.StorageElement.StorageElementItem", new=Mock())
  @patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations', new=Mock())
  @patch("DIRAC.Resources.Storage.StorageElement.StorageElementItem", new=Mock())
  @patch("DIRAC.Resources.Storage.StorageFactory.StorageFactory", new=Mock())
  def setUp(self):  # pylint: disable=R0915
    """Set up the objects."""
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)

    self.log = gLogger.getSubLogger("MODULEBASE")

    self.prod_id = 123
    self.prod_job_id = 456
    self.wms_job_id = 0
    self.workflowStatus = {'OK': True}
    self.stepStatus = {'OK': True}

    self.jr_mock = Mock()
    self.jr_mock.setApplicationStatus.return_value = {'OK': True, 'Value': ''}
    self.jr_mock.generateRequest.return_value = {'OK': True, 'Value': 'pippo'}
    self.jr_mock.setJobParameter.return_value = {'OK': True, 'Value': 'pippo'}
    self.jr_mock.generateForwardDISET.return_value = {'OK': True, 'Value': 'pippo'}
#    self.jr_mock.setJobApplicationStatus.return_value = {'OK': True, 'Value': 'pippo'}

    self.fr_mock = Mock()
    self.fr_mock.getFiles.return_value = {
        "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep": "ApplicationCrash",
        "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep": "ARG"}
    self.fr_mock.setFileStatus.return_value = {'OK': True, 'Value': ''}
    self.fr_mock.commit.return_value = {'OK': True, 'Value': ''}
    self.fr_mock.generateRequest.return_value = {'OK': True, 'Value': ''}

    self.rm_mock = Mock()
    self.rm_mock.getReplicas.return_value = {'OK': True, 'Value': {'Successful': {'pippo': 'metadataPippo'},
                                                                   'Failed': None}}
    self.rm_mock.getCatalogFileMetadata.return_value = {'OK': True, 'Value': {'Successful': {'pippo': 'metadataPippo'},
                                                                              'Failed': None}}
    self.rm_mock.removeFile.return_value = {'OK': True, 'Value': {'Failed': False}}
    self.rm_mock.putStorageDirectory.return_value = {'OK': True, 'Value': {'Failed': False}}
    self.rm_mock.addCatalogFile.return_value = {'OK': True, 'Value': {'Failed': False}}
    self.rm_mock.putAndRegister.return_value = {'OK': True, 'Value': {'Failed': False}}
    self.rm_mock.getFile.return_value = {'OK': True, 'Value': {'Failed': False}}

    self.rc_mock = Mock(name='RequestContainer')
    self.rc_mock.update.return_value = {'OK': True, 'Value': ''}
    self.rc_mock.setDISETRequest.return_value = {'OK': True, 'Value': ''}
    self.rc_mock.isEmpty.return_value = {'OK': True, 'Value': ''}
    self.rc_mock.toXML.return_value = {'OK': True, 'Value': 'Ex Em El'}
    self.rc_mock.getDigest.return_value = {'OK': True, 'Value': 'Indigestion'}
    self.rc_mock.toJSON.return_value = S_OK("JSON Bieber requests your presence")
    self.rc_mock.__len__.return_value = 1

    self.ar_mock = Mock()
    self.ar_mock.commit.return_value = {'OK': True, 'Value': ''}

    self.wf_commons = [{'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'eventType': '123456789',
                        'jobType': 'merge',
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'numberOfEvents': '100',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'merge',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'numberOfEvents': '100',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'merge',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'numberOfEvents': '100',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'merge',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'numberOfEvents': '100',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'InputData': '',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'InputData': 'foo;bar',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'InputData': 'foo;bar',
                        'ParametricInputData': '',
                        'appSteps': ['someApp_1']},
                       {'PRODUCTION_ID': str(self.prod_id),
                        'JOB_ID': str(self.prod_job_id),
                        'configName': 'aConfigName',
                        'configVersion': 'aConfigVersion',
                        'outputDataFileMask': '',
                        'jobType': 'reco',
                        'BookkeepingLFNs': 'aa',
                        'ProductionOutputData': 'ProductionOutputData',
                        'JobReport': self.jr_mock,
                        'Request': self.rc_mock,
                        'AccountingReport': self.ar_mock,
                        'FileReport': self.fr_mock,
                        'SystemConfig': 'sys_config',
                        'LogFilePath': 'someDir',
                        'LogTargetPath': 'someOtherDir',
                        'runNumber': 'Unknown',
                        'InputData': 'foo;bar',
                        'ParametricInputData': 'pid1;pid2;pid3',
                        'appSteps': ['someApp_1']},
        ]
    self.step_commons = [{'applicationName': 'someApp',
                          'applicationVersion': 'v1r0',
                          'eventType': '123456789',
                          'applicationLog': 'appLog',
                          'extraPackages': '',
                          'XMLSummary': 'XMLSummaryFile',
                          'numberOfEvents': '100',
                          'BKStepID': '123',
                          'StepProcPass': 'Sim123',
                          'outputFilePrefix': 'pref_',
                          'STEP_INSTANCE_NAME': 'someApp_1',
                          'listoutput': [{'outputDataName': str(self.prod_id) + '_' + str(self.prod_job_id) + '_',
                                          'outputDataSE': 'aaa',
                                          'outputDataType': 'bbb'}]},
                         {'applicationName': 'someApp',
                          'applicationVersion': 'v1r0',
                          'eventType': '123456789',
                          'applicationLog': 'appLog',
                          'extraPackages': '',
                          'XMLSummary': 'XMLSummaryFile',
                          'numberOfEvents': '100',
                          'BKStepID': '123',
                          'StepProcPass': 'Sim123',
                          'outputFilePrefix': 'pref_',
                          'optionsLine': '',
                          'STEP_INSTANCE_NAME': 'someApp_1',
                          'listoutput': [{'outputDataName': str(self.prod_id) + '_' + str(self.prod_job_id) + '_',
                                          'outputDataSE': 'aaa',
                                          'outputDataType': 'bbb'}]},
                         {'applicationName': 'someApp',
                          'applicationVersion': 'v1r0',
                          'eventType': '123456789',
                          'applicationLog': 'appLog',
                          'extraPackages': '',
                          'XMLSummary': 'XMLSummaryFile',
                          'numberOfEvents': '100',
                          'BKStepID': '123',
                          'StepProcPass': 'Sim123',
                          'outputFilePrefix': 'pref_',
                          'extraOptionsLine': 'blaBla',
                          'STEP_INSTANCE_NAME': 'someApp_1',
                          'listoutput': [{'outputDataName': str(self.prod_id) + '_' + str(self.prod_job_id) + '_',
                                          'outputDataSE': 'aaa',
                                          'outputDataType': 'bbb'}]}]
    self.step_number = '321'
    self.step_id = '%s_%s_%s' % (self.prod_id, self.prod_job_id, self.step_number)

    self.mbase = ModuleBase()
    self.mbase.rm = self.rm_mock
    self.mbase.request = self.rc_mock
    self.mbase.jobReport = self.jr_mock
    self.mbase.fileReport = self.fr_mock
    self.mbase.workflow_commons = self.wf_commons[0]
    self.mbase.workflow_commons['LogFilePath'] = "/ilc/user/s/sailer/test/dummy/folder"
    self.mbase.workflow_commons['Platform'] = "x86_64-slc5-gcc43-opt"
    self.mbase.log = gLogger.getSubLogger("ModuleBaseTest")
    self.mbase.log.showHeaders(True)
    self.mbase.ignoreapperrors = False
    self.uod = UploadOutputData()
    self.uod.workflow_commons = self.mbase.workflow_commons

    self.ulf = UploadLogFile()

    # create some dummy files
    for i in range(0, 8):
      path = "h_nunu_gen_4191_000%s.stdhep" % str(i)
      with open(path, 'a'):
        pass
    path = "test3.stdhep"
    with open(path, 'a'):
      pass
    try:
      os.makedirs("myILDConfig")
    except OSError:
      pass

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)


@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
class TestModuleBase(ModulesTestCase):
  """Tests for ModuleBase functions."""

  def test_generateFailoverFile(self):
    """run the generateFailoverFile function and see what happens..................................."""
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      dummy_res = self.mbase.generateFailoverFile()
    #print res

  def test_CreateRemoveRequest(self):
    """ModuleBase: Create a removal request for some LFN............................................"""
    gLogger.setLevel("ERROR")
    lfnList = ['/ilc/user/s/sailer/2014_11/12/12345/testsim.slcio', '/ilc/user/s/sailer/2014_11/12/12345/testsim.slcio']
    mob = ModuleBase()
    mob.workflow_commons = dict()
    mob.jobID = 444444
    mob.addRemovalRequests(lfnList)
    request = mob.workflow_commons['Request']
    self.assertEqual(len(request), 1)

  def test_MB_getCandidateFiles(self):
    """ModuleBase: getCandidateFiles: files exist..................................................."""
    gLogger.setLevel("ERROR")
    outputList = list({
        'h_nunu_gen_4191_0007': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0007.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0006': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0006.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0005': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0005.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0004': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0004.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0003': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0003.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0002': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0002.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0001': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0001.stdhep',
            'outputDataSE': 'CERN-SRM'},
        'h_nunu_gen_4191_0000': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_0000.stdhep',
            'outputDataSE': 'CERN-SRM'}}.values())
    outputLFNs = ['/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0004.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0005.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0006.stdhep',
                  '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0007.stdhep']
    dummy_fileMask = None
    result = self.mbase.getCandidateFiles(outputList, outputLFNs, dummy_fileMask)
    print(result)
    resDict = [os.path.basename(lfn) in result['Value'] for lfn in outputLFNs]
    gLogger.debug("Result: %s" % result)
    gLogger.debug("ResDict: %s" % resDict)
    self.assertTrue(all(resDict))

  def test_MB_getCandidateFiles_FileNotFound(self):
    """ModuleBase: getCandidateFiles: No Such File.................................................."""
    gLogger.setLevel("ERROR")
    outputList = list({
        'h_nunu_gen_4191_NSF': {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'h_nunu_gen_4191_NSF.stdhep',
            'outputDataSE': 'CERN-SRM'}}.values())
    outputLFNs = ['/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_NSF.stdhep']
    dummy_fileMask = None
    result = self.mbase.getCandidateFiles(outputList, outputLFNs, dummy_fileMask)
    self.assertIn("Output Data Not Found", result['Message'])

  def test_MB_getCandidateFiles_FileTooLong(self):
    """ModuleBase: getCandidateFiles: File Too Long................................................."""
    gLogger.setLevel("ERROR")
    outputList = list({
        'a'
        * 128: {
            'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
            'outputFile': 'a'
            * 128,
            'outputDataSE': 'CERN-SRM'}}.values())
    outputLFNs = ['/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/' + 'a' * 128]
    dummy_fileMask = None
    result = self.mbase.getCandidateFiles(outputList, outputLFNs, dummy_fileMask)
    self.assertIn("Filename too long", result['Message'])

  def test_MB_getCandidateFiles_PathTooLong(self):
    """ModuleBase: getCandidateFiles: Path Too Long................................................."""
    gLogger.setLevel("ERROR")
    outputList = list({
        'a'
        * 127: {
            'outputPath': '/bbbbbbbbbb'
            * 26,
            'outputFile': 'a'
            * 127,
            'outputDataSE': 'CERN-SRM'}}.values())
    outputLFNs = ['/bbbbbbbbbb' * 26 + '/' + 'a' * 127]
    dummy_fileMask = None
    result = self.mbase.getCandidateFiles(outputList, outputLFNs, dummy_fileMask)
    self.assertIn("LFN too long", result['Message'])

  def test_MB_logWorkingDirectory(self):
    """ModuleBase: logWorkingDirectory.............................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.logWorkingDirectory()

  def test_MB_redirectLogOutput_1(self):
    """ModuleBase: redirectLogOutput 1.............................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = "+++ Event String"
    message = "+++ Event String 123"
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      self.mbase.redirectLogOutput(fd, message)
    self.assertEqual(message, out.getvalue().strip().splitlines()[0])

  def test_MB_redirectLogOutput_2(self):
    """ModuleBase: redirectLogOutput 2.............................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = "+++ Event String"
    self.mbase.applicationLog = "grailDiary.log"
    message = ["+++ Event String 123", "andSomeOtherString"]
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      for mes in message:
        self.mbase.redirectLogOutput(fd, mes)
    with open(self.mbase.applicationLog, "r") as logF:
      self.assertEqual(logF.read().strip(), "\n".join(message))
    self.assertEqual(message[0], out.getvalue().strip().splitlines()[0])

  def test_MB_redirectLogOutput_3(self):
    """ModuleBase: redirectLogOutput 3.............................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = "+++ Event String"
    self.mbase.applicationLog = "grailDiary.log"
    self.mbase.excludeAllButEventString = True
    message = ["+++ Event String 123", "andSomeOtherString"]
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      for mes in message:
        self.mbase.redirectLogOutput(fd, mes)
    with open(self.mbase.applicationLog, "r") as logF:
      self.assertEqual(logF.read().strip(), message[0])
    self.assertEqual(message[0], out.getvalue().strip().splitlines()[0])

  def test_MB_redirectLogOutput_4(self):
    """ModuleBase: redirectLogOutput 4.............................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = ""
    self.mbase.applicationLog = "grailDiary.log"
    self.mbase.excludeAllButEventString = True
    message = ["+++ Event String 123", "andSomeOtherString"]
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      for mes in message:
        self.mbase.redirectLogOutput(fd, mes)
    with open(self.mbase.applicationLog, "r") as logF:
      self.assertEqual(logF.read().strip(), "")
    self.assertEqual("", out.getvalue().strip())

  def test_MB_redirectLogOutput_noMes(self):
    """ModuleBase: redirectLogOutput no message....................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = "+++ Event String"
    self.mbase.applicationLog = "grailDiary.log"
    message = ""
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      self.mbase.redirectLogOutput(fd, message)
    self.assertFalse(os.path.exists(self.mbase.applicationLog))
    self.assertEqual("", out.getvalue().strip())

  def test_MB_redirectLogOutput_noES(self):
    """ModuleBase: redirectLogOutput no eventstring................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = []
    self.mbase.applicationLog = "grailDiary.log"
    message = "some message"
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      self.mbase.redirectLogOutput(fd, message)
    with open(self.mbase.applicationLog, "r") as logF:
      self.assertEqual(logF.read().strip(), message)
    self.assertEqual("", out.getvalue().strip())

  def test_MB_redirectLogOutput_noES_2(self):
    """ModuleBase: redirectLogOutput no eventstring 2..............................................."""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = ''
    self.mbase.applicationLog = "grailDiary.log"
    message = "some message"
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      self.mbase.redirectLogOutput(fd, message)
    with open(self.mbase.applicationLog, "r") as logF:
      self.assertEqual(logF.read().strip(), message)
    self.assertEqual("", out.getvalue().strip())

  def test_MB_redirectLogOutput_ESNone(self):
    """ModuleBase: redirectLogOutput eventstring is None............................................"""
    gLogger.setLevel("ERROR")
    self.mbase.eventstring = None
    self.mbase.applicationLog = "grailDiary.log"
    message = ["some message", "and some other message"]
    out = StringIO()
    sys.stdout = out
    with open("logFile", "w") as fd:
      for mes in message:
        self.mbase.redirectLogOutput(fd, mes)
    with open(self.mbase.applicationLog, "r") as logF:
      self.assertEqual(logF.read().strip().splitlines(), message)
    self.assertEqual(message, out.getvalue().strip().splitlines())

  def test_MB_treatConfigPackage(self):
    """ModuleBase: treatConfigPackage..............................................................."""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['ILDConfigPackage'] = "ILDConfigv01-16-p03"
    with patch("ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation.checkCVMFS",
                Mock(return_value=S_OK(("myILDConfig", "init.sh")))  # needs tuple
              ):
      res = self.mbase.treatConfigPackage()
    self.assertTrue(res['OK'])

  def test_MB_resolveParametricInputData_1(self):
    """ModuleBase: resolveInputVariables parametric list success ..................................."""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['ParametricInputData'] = ["LFN:/first/file/path.ext", "LFN:/second/file/path.ext"]
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveParametricInputData_2(self):
    """ModuleBase: resolveInputVariables parametric string success ................................."""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['ParametricInputData'] = "LFN:/first/file/path.ext;LFN:/second/file/path.ext"
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveParametricInputData_3(self):
    """ModuleBase: resolveInputVariables parametric list success no lfn............................."""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['ParametricInputData'] = ["/first/file/path.ext", "/second/file/path.ext"]
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveParametricInputData_4(self):
    """ModuleBase: resolveInputVariables parametric string success no lfn..........................."""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['ParametricInputData'] = "/first/file/path.ext;/second/file/path.ext"
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveInputData_1(self):
    """ModuleBase: resolveInputVariables list success .............................................."""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['InputData'] = ["LFN:/first/file/path.ext", "LFN:/second/file/path.ext"]
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveInputData_2(self):
    """ModuleBase: resolveInputVariables string success ............................................"""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['InputData'] = "LFN:/first/file/path.ext;LFN:/second/file/path.ext"
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveInputData_3(self):
    """ModuleBase: resolveInputVariables list success no lfn........................................"""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['InputData'] = ["/first/file/path.ext", "/second/file/path.ext"]
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)

  def test_MB_resolveInputData_4(self):
    """ModuleBase: resolveInputVariables string success no lfn......................................"""
    gLogger.setLevel("ERROR")
    self.mbase.platform = self.mbase.workflow_commons.get('Platform', self.mbase.platform)
    self.mbase.workflow_commons['InputData'] = "/first/file/path.ext;/second/file/path.ext"
    res = self.mbase.resolveInputVariables()
    self.assertTrue(res['OK'])
    self.assertEqual(["/first/file/path.ext", "/second/file/path.ext"], self.mbase.InputData)


#############################################################################
# UploadLogFile.py
#############################################################################

@patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("DIRAC.Resources.Storage.StorageElement.StorageElementItem", new=Mock())
@patch("DIRAC.Resources.Storage.StorageFactory.StorageFactory", new=Mock())
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
@patch("%s.UploadLogFile.StorageElement" % MODULE_NAME, new=Mock(return_value=S_OK()))
class TestUploadLogFile(ModulesTestCase):
  """test UploadLogFile."""

  @patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
  @patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
  def setUp(self):
    """create logfile."""
    super(TestUploadLogFile, self).setUp()
    with open("MyLogFile.log", "w") as logFile:
      logFile.write("soemthing")
    with open("MyOtherLogFile.log", "w") as logFile:
      logFile.write("soemthing")
    try:
      os.makedirs("./my/log/folder")
    except OSError:
      pass
    with open("./my/log/folder/MyLogFile.log", "w") as logFile:
      logFile.write("something else")

  def test_ULF_ASI_NoLogFiles(self):
    """ULF.applicationSpecificInputs: no log files present.........................................."""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.log = gLogger.getSubLogger("ULF-NoLogFiles")
    self.ulf.log.setLevel("INFO")

    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    self.ulf.applicationSpecificInputs()
    res = self.ulf.execute()
    self.assertTrue(res['OK'])

  def test_ULF_ASI_enableNotBool(self):
    """ULF.applicationSpecificInputs: enable is not boolean........................................."""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.enable = "notboolean"
    self.ulf.failoverTest = "notboolean"
    self.ulf.jobID = 0
    self.ulf.log = gLogger.getSubLogger("ULF-NoLogFiles")
    self.ulf.log.setLevel("INFO")
    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    self.ulf.applicationSpecificInputs()
    res = self.ulf.execute()
    self.assertTrue(res['OK'])

  @patch("%s.UploadLogFile.getLogPath" % MODULE_NAME, new=Mock(return_value=S_ERROR("not the path you are looking for")))
  def test_ULF_ASI_noLogPath(self):
    """ULF.applicationSpecificInputs: getLogPath fails.............................................."""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.workflow_commons.pop("LogFilePath", None)
    self.ulf.workflow_commons.pop("LogTargetPath", None)
    self.ulf.log = gLogger.getSubLogger("ULF-NoLogFiles")
    self.ulf.log.setLevel("INFO")
    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    res = self.ulf.applicationSpecificInputs()
    self.assertEqual(res['Message'], "not the path you are looking for")

  def test_ULF_ASI_ListLogPath(self):
    """ULF.applicationSpecificInputs: getLogPath does not return strings............................"""
    self.ulf = UploadLogFile()
    self.ulf.logFilePath = None
    self.ulf.logTargetPath = None
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.workflow_commons["LogFilePath"] = ["/ilc/prod/ilc/sid", "path"]
    self.ulf.workflow_commons["LogTargetPath"] = ["/ilc/prod/clic/log.tar.gz", "path"]
    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    _res = self.ulf.applicationSpecificInputs()
    self.assertEqual(self.ulf.experiment, "ILC_SID")

  def test_ULF_ASI_expClic(self):
    """ULF.applicationSpecificInputs: experiment CLIC..............................................."""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.workflow_commons["LogFilePath"] = "/ilc/prod/clic"
    self.ulf.workflow_commons["LogTargetPath"] = "/ilc/prod/clic/log.tar.gz"
    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    _res = self.ulf.applicationSpecificInputs()
    self.assertEqual(self.ulf.experiment, "CLIC")

  def test_ULF_ASI_expSid(self):
    """ULF.applicationSpecificInputs: experiment Sid................................................"""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.workflow_commons["LogFilePath"] = "/ilc/prod/ilc/sid/prodID/gen"
    self.ulf.workflow_commons["LogTargetPath"] = "/ilc/prod/ilc/sid/log.tar.gz"
    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    _res = self.ulf.applicationSpecificInputs()
    self.assertEqual(self.ulf.experiment, "ILC_SID")

  def test_ULF_ASI_expILD(self):
    """ULF.applicationSpecificInputs: experiment ILD................................................"""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.log = gLogger.getSubLogger("ULF-NoLogFiles")
    self.ulf.log.setLevel("INFO")
    self.ulf.workflow_commons["LogFilePath"] = "/ilc/prod/ilc/mc-dbd"
    self.ulf.workflow_commons["LogTargetPath"] = "/ilc/prod/ilc/mc-dbd/log.tar.gz"
    self.ulf.resolveInputVariables = Mock(return_value=S_OK())
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK([]))
    _res = self.ulf.applicationSpecificInputs()
    self.assertEqual(self.ulf.experiment, "ILC_ILD")

  def test_ULF_ASI_OneLogFile(self):
    """ULF.applicationSpecificInputs: one log files present........................................."""
    self.ulf = UploadLogFile()
    self.ulf.log = gLogger.getSubLogger("ULF-OneLogFile")
    self.ulf.workflow_commons = copy.deepcopy(self.wf_commons[0])
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK(['MyLogFile.log']))
    self.ulf.logSE = Mock()
    self.ulf.logSE.putFile = Mock(return_value=S_OK(dict(Failed=['MyLogFiles.tar.gz'])))
    self.ulf.logLFNPath = getLogPath(self.ulf.workflow_commons)['Value']['LogTargetPath'][0]
    self.ulf._tryFailoverTransfer = Mock(return_value=S_OK({'Request': self.rc_mock,
                                                              'uploadedSE': 'CERN-SRM'}))
    self.ulf.applicationSpecificInputs()
    res = self.ulf.execute()
    self.assertTrue(res['OK'])

  def test_ULF_ASI_FailedFailover(self):
    """ULF.applicationSpecificInputs: Failovertransfer fails........................................"""
    gLogger.setLevel("ERROR")
    self.ulf = UploadLogFile()
    self.ulf.log = gLogger.getSubLogger("ULF-OneLogFile")
    self.ulf.workflow_commons = copy.deepcopy(self.wf_commons[0])
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK(['MyLogFile.log']))
    self.ulf.logSE = Mock()
    self.ulf.logSE.putFile = Mock(return_value=S_OK(dict(Failed=['MyLogFiles.tar.gz'])))
    self.ulf.logLFNPath = getLogPath(self.ulf.workflow_commons)['Value']['LogTargetPath'][0]
    self.ulf._tryFailoverTransfer = Mock(return_value=S_OK())
    self.ulf.applicationSpecificInputs()
    res = self.ulf.execute()
    self.assertTrue(res['OK'])

  def test_ULF_ASI_FailedputFile(self):
    """ULF.applicationSpecificInputs: putfile failed completely....................................."""
    gLogger.setLevel("ERROR")
    self.ulf = UploadLogFile()
    self.ulf.log = gLogger.getSubLogger("ULF-OneLogFile")
    self.ulf.workflow_commons = copy.deepcopy(self.wf_commons[0])
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK(['MyLogFile.log']))
    self.ulf.logSE = Mock()
    self.ulf.logSE.putFile = Mock(return_value=S_ERROR("Ekke Ekke Ekke Ekke"))
    self.ulf.logLFNPath = getLogPath(self.ulf.workflow_commons)['Value']['LogTargetPath'][0]
    self.ulf._tryFailoverTransfer = Mock(return_value=S_OK())
    self.ulf.applicationSpecificInputs()
    res = self.ulf.execute()
    self.assertTrue(res['OK'])

  def test_ULF_ASI_LogFileGone(self):
    """ULF.applicationSpecificInputs: log file disappeared, IOError................................."""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.wf_commons[0])
    self.ulf.log = gLogger.getSubLogger("ULF-LogFileGone")

    self.ulf.logLFNPath = getLogPath(self.ulf.workflow_commons)['Value']['LogTargetPath'][0]
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK(['std.out']))
    res = self.ulf.execute()
    assert res['OK']
    assert "Empty Log Directory" in res['Value']

  def test_ULF_ASI_execute(self):
    """ULF.ASI,Exe: run through and get request....................................................."""
    self.ulf.workflow_commons = copy.deepcopy(self.wf_commons[0])
    self.ulf.log = gLogger.getSubLogger("ULF-RequestTest")
    self.ulf.jobID = 12345
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK(['MyLogFile.log', 'MyOtherLogFile.log']))
    self.ulf.logSE = Mock()
    self.ulf.logSE.putFile = Mock(return_value=S_OK(dict(Failed=['MyLogFile.log', 'MyOtherLogFile.log'],
                                                         Message="Ekke Ekke Ekke Ekke")))
    self.mbase.workflow_commons['Request'] = Request()
    self.mbase.workflow_commons['Request'].RequestName = "MockingRequest"
    self.ulf._tryFailoverTransfer = Mock(return_value=S_OK({'Request': self.mbase.workflow_commons['Request'],
                                                              'uploadedSE': 'CERN-SRM'}))
    self.ulf._getRequestContainer = self.rc_mock
    self.ulf.logLFNPath = getLogPath(self.ulf.workflow_commons)['Value']['LogTargetPath'][0]
    self.ulf.applicationSpecificInputs()
    res = self.ulf.execute()
    self.assertTrue(res['OK'])

  def test_ULF_finalize_move(self):
    """ULF.Finalize: move to new folder............................................................."""
    gLogger.setLevel("ERROR")
    with patch("DIRAC.Resources.Storage.StorageElement.StorageElementItem", Mock()):
      self.ulf = UploadLogFile()
    self.ulf.logSE = Mock()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.log = gLogger.getSubLogger("ULF-FinalMove")
    self.ulf.jobID = 12345
    self.ulf._determineRelevantFiles = Mock(return_value=S_OK(['MyLogFile.log', 'MyOtherLogFile.log']))
    # self.ulf.logSE.putFile = Mock(return_value=S_OK(dict(Failed=['MyLogFiles.tar.gz'],
    #                                                     Message="Ekke Ekke Ekke Ekke")))
    self.mbase.workflow_commons['Request'] = self.rc_mock
    self.ulf._tryFailoverTransfer = Mock(return_value=S_OK({'Request': self.mbase.workflow_commons['Request'],
                                                              'uploadedSE': 'CERN-SRM'}))
    self.ulf.logLFNPath = getLogPath(self.ulf.workflow_commons)['Value']['LogTargetPath'][0]
    self.ulf.logFilePath = self.ulf.workflow_commons['LogFilePath']
    self.ulf.logdir = os.path.realpath("./my/log/folder")
    # self.ulf.execute()
    res = self.ulf.finalize()
    self.assertTrue(res['OK'])

  def test_ULF__determineRelevantFiles_1(self):
    """ULF._determineRelevantFiles 1................................................................"""
    self.ulf = UploadLogFile()
    res = self.ulf._determineRelevantFiles()
    self.assertTrue(res['OK'])

  @patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue', new=Mock(return_value=["*.root"]))
  def test_ULF__determineRelevantFiles_2(self):
    """ULF._determineRelevantFiles 2................................................................"""
    self.ulf = UploadLogFile()
    _res = self.ulf._determineRelevantFiles()
    self.assertEqual(self.ulf.logExtensions, ["*.root"])

  @patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations.getValue', new=Mock(return_value=["FAIL-SRM"]))
  @patch('%s.UploadLogFile.FailoverTransfer' % MODULE_NAME, new=Mock())
  def test_ULF__tryFailoverTransfer_1(self):
    """ULF._tryFailoverTransfer 1..................................................................."""
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.workflow_commons['Request'] = self.rc_mock
    _res = self.ulf._tryFailoverTransfer("log.tar.gz", "myLogDir")
    self.assertEqual(self.ulf.failoverSEs, ["FAIL-SRM"])

  def test_ULF__tryFailoverTransfer_2(self):
    """ULF._tryFailoverTransfer 2..................................................................."""

    from DIRAC.DataManagementSystem.Client.FailoverTransfer import FailoverTransfer
    FailoverTransfer.transferAndRegisterFile = Mock(return_value=S_ERROR("IT ACTUALLY WORKS!!!!!!1eleven!!"))
    self.ulf = UploadLogFile()
    self.ulf.workflow_commons = copy.deepcopy(self.mbase.workflow_commons)
    self.ulf.workflow_commons['Request'] = self.rc_mock
    res = self.ulf._tryFailoverTransfer("log.tar.gz", "myLogDir")
    self.assertEqual(res['Value'], None)

#############################################################################
# UploadOutputData.py
#############################################################################


@patch('ILCDIRAC.Core.Utilities.ProductionData.Operations', new=createOperationsMock())
@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
class UploadOutputDataSuccess(ModulesTestCase):
  """test UploadLogFile."""

  def test_execute(self):
    """tests execute function......................................................................."""
    pass


@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
class UploadOutputDataFailure(ModulesTestCase):
  """test UploadLogFile."""

  def test_execute(self):
    """tests execute function......................................................................."""
    pass

#############################################################################
# FailoverRequest.py
#############################################################################


@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("DIRAC.Resources.Storage.StorageElement.StorageElementItem", new=Mock())
@patch("DIRAC.Resources.Storage.StorageFactory.StorageFactory", new=Mock())
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
@patch("%s.UploadLogFile.StorageElement" % MODULE_NAME, new=Mock(return_value=S_OK()))
@patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations', new=Mock())
class TestFailoverRequest(ModulesTestCase):
  """test UploadLogFile."""

  @patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
  @patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
  def setUp(self):
    super(TestFailoverRequest, self).setUp()
    self.frq = None

  @patch("DIRAC.RequestManagementSystem.private.RequestValidator.RequestValidator", new=Mock(return_value=S_OK()))
  @patch("DIRAC.RequestManagementSystem.private.RequestValidator.RequestValidator.validate", new=Mock(return_value=S_OK()))
  def test_ASI_Enabled(self):
    """applicationSpecificInputs: control flag is enabled..........................................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.workflow_commons = dict()
    self.frq.log = gLogger.getSubLogger("testASI")
    os.environ['JOBID'] = "12345"
    self.frq.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertTrue(self.frq.enable)

  @patch("DIRAC.RequestManagementSystem.private.RequestValidator.RequestValidator", new=Mock(return_value=S_OK()))
  @patch("DIRAC.RequestManagementSystem.private.RequestValidator.RequestValidator.validate", new=Mock(return_value=S_OK()))
  def test_ASI_Disable(self):
    """applicationSpecificInputs: control flag is enabled with non boolean.........................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.workflow_commons = dict()
    self.frq.log = gLogger.getSubLogger("testASI")
    os.environ['JOBID'] = "12345"
    self.frq.step_commons = dict(Enable="arg")
    self.frq.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertFalse(self.frq.enable)

  def test_ASI_Disabled(self):
    """applicationSpecificInputs: control flag is disabled.........................................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.workflow_commons = dict()
    self.frq.log = gLogger.getSubLogger("testASI")

    self.frq.applicationSpecificInputs()
    self.assertFalse(self.frq.enable)

  def test_ASI_AllVariables(self):
    """applicationSpecificInputs: checks if all variables have been properly set after this call...."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    os.environ['JOBID'] = "12345"
    self.frq.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertTrue(self.frq.jobReport and self.frq.fileReport
                     and self.frq.productionID and self.frq.prodJobID and self.frq.enable)

  def test_ASI_NoVariables(self):
    """applicationSpecificInputs: checks that no variables have been set after this call............"""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.workflow_commons = dict()
    os.environ['JOBID'] = "12345"
    self.frq.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertFalse(self.frq.jobReport or self.frq.fileReport
                      or self.frq.productionID or self.frq.prodJobID)

  def test_Exe_Disabled(self):
    """execute: is disabled........................................................................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq._getJobReporter = Mock(return_value=self.jr_mock)
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Disabled")
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.enable = False
    self.frq.workflow_commons = dict()
    res = self.frq.execute()
    self.assertIn("Module is disabled", res['Value'])

  def test_Exe_WFFail(self):
    """execute: WF Failed..........................................................................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Fail")
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.jr_mock.generateForwardDISET = Mock(return_value=S_ERROR("EKKE"))
    self.frq.enable = True
    self.frq.jobID = 12345
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflowStatus = S_ERROR()
    res = self.frq.execute()
    self.assertFalse(res['OK'])

  def test_Exe_RIV_Failes(self):
    """execute: WF Failed..........................................................................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Fail")
    self.frq.resolveInputVariables = Mock(return_value=S_ERROR("EKKE: no input variables"))
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.jr_mock.generateForwardDISET = Mock(return_value=S_ERROR("EKKE"))
    self.frq.enable = True
    self.frq.jobID = 12345
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflowStatus = S_ERROR()
    res = self.frq.execute()
    self.assertFalse(res['OK'])

  def test_Exe_Success(self):
    """execute: succeeds............................................................................"""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(res['OK'])

  def test_Exe_Success_input(self):
    """execute: succeeds inputdata.................................................................."""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.fileReport = self.fr_mock
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.InputData = ["/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep"]
    self.frq.productionID = 43321
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(res['OK'])

  def test_Exe_Success_input_no(self):
    """execute: succeeds inputdata.................................................................."""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.fileReport = self.fr_mock
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.InputData = ["/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep",
                         ]
    self.frq.productionID = 43321
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(res['OK'])

  def test_Exe_statusFalse(self):
    """execute: workflowStatus not OK..............................................................."""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.fileReport = self.fr_mock
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.InputData = ["/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep",
                         ]
    self.frq.productionID = 43321
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.workflowStatus['OK'] = False
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(not res['OK'])

  def test_Exe_statusFalse_diset_err(self):
    """execute: workflowStatus not OK diset error..................................................."""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.InputData = ["/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep",
                         ]
    self.frq.productionID = 43321
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.workflowStatus['OK'] = False
    self.frq.jobType = "NotSplit"
    fr_mock = Mock(self.fr_mock)
    fr_mock.commit.return_value = S_ERROR()
    fr_mock.generateForwardDISET = Mock(return_value=S_ERROR("EKKE"))
    self.frq.fileReport = fr_mock
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(not res['OK'])

  def test_Exe_statusFalse_diset_ok(self):
    """execute: workflowStatus not OK diset success................................................."""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.InputData = ["/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep",
                         ]
    self.frq.productionID = 43321
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.workflowStatus['OK'] = False
    fr_mock = Mock(self.fr_mock)
    fr_mock.commit.return_value = S_ERROR()
    fr_mock.generateForwardDISET = Mock(return_value=S_OK(None))
    self.frq.fileReport = fr_mock
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(not res['OK'])

  def test_Exe_statusFalse_failoverFail(self):
    """execute: workflowStatus not OK failoverfailure..............................................."""
    gLogger.setLevel("ERROR")
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-Succeed")
    self.frq.InputData = ["/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep",
                          "/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep",
                         ]
    self.frq.productionID = 43321
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.jobID = 12345
    self.frq.generateFailoverFile = Mock(return_value=S_ERROR("No can do"))
    self.frq.workflowStatus['OK'] = False
    fr_mock = Mock(self.fr_mock)
    fr_mock.commit.return_value = S_ERROR()
    fr_mock.generateForwardDISET = Mock(return_value=S_OK(None))
    self.frq.fileReport = fr_mock
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=fr_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    self.frq.workflow_commons['Request'] = self.rc_mock
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      res = self.frq.execute()
    self.assertTrue(not res['OK'])

  def test_Exe_genDisetRequest(self):
    """execute: Generate Diset Request.............................................................."""
    gLogger.setLevel("ERROR")
    self.frq = FailoverRequest()
    self.frq.log = gLogger.getSubLogger("Frq-Exe-GenDiset")
    self.frq.applicationSpecificInputs = Mock(return_value=S_OK())
    self.frq.enable = True
    self.frq.jobID = 12345
    self.frq.fileReport = Mock(name="FailedFileReport")
    self.frq.fileReport.commit.return_value = S_ERROR("Nobody suspects the ")
    self.frq.fileReport.generateForwardDISET.return_value = S_OK("Spanish Inquisition")
    self.frq.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.frq.fileReport,
                                      Request=self.rc_mock, PRODUCTION_ID=43321, JOB_ID=12345)
    with patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, Mock()):
      self.frq.execute()
    self.assertTrue(self.frq.workflow_commons['Request'])

  def test_set_registrationRequest(self):
    """execute: test if setRegistrationRequest succeeds ............................................"""
    # setup the filedict from getFileMetaData and pass that to setRegistrationRequest
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.prodOutputLFNs = ['/ilc/user/s/sailer/test3.stdhep']
    self.uod.jobReport = Mock()
    self.uod.jobReport.generateForwardDISET.return_value = S_ERROR("No JobRep")
    self.uod.fileReport = self.fr_mock
    self.uod.workflow_commons = dict(Enable=True, PRODUCTION_ID=43321, JOB_ID=12345)

    candidateFiles = {'test3.stdhep': {'lfn': '/ilc/user/s/sailer/test3.stdhep', 'workflowSE': 'CERN-DIP-4'}}
    self.uod.getCandidateFiles = Mock(return_value=S_OK())
    res = self.uod.getFileMetadata(candidateFiles)
    fileDict = res['Value']
    self.uod.getFileMetadata = Mock(return_value=res)
    self.uod.getDestinationSEList = Mock(return_value=S_OK(["CERN-DIP-4"]))
    self.uod.enable = True
    self.uod.jobID = 12345
    from DIRAC.DataManagementSystem.Client.FailoverTransfer import FailoverTransfer
    lfn = '/ilc/user/s/sailer/test3.stdhep'
    targetSE = "CERN-DIP-4"
    catalog = ["FileCatalog"]
    fot = FailoverTransfer(self.uod._getRequestContainer())
    fot._setRegistrationRequest(lfn, targetSE, fileDict['test3.stdhep']['filedict'], catalog)
    res = self.uod.generateFailoverFile()

#############################################################################
# UploadOutputData.py
#############################################################################


@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
class TestUploadOutputData(ModulesTestCase):
  """test UploadOutputData."""

  @patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
  @patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
  def setUp(self):
    super(TestUploadOutputData, self).setUp()
    self.uod = None

  def test_ASI_Enabled(self):
    """UOD.applicationSpecificInputs: control flag is enabled......................................."""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = dict()
    os.environ['JOBID'] = "12345"
    self.uod.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertTrue(self.uod.enable)

  def test_ASI_Disable(self):
    """UOD.applicationSpecificInputs: control flag is enabled with non boolean......................"""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = dict()
    os.environ['JOBID'] = "12345"
    self.uod.step_commons = dict(Enable="arg")
    self.uod.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertFalse(self.uod.enable)

  def test_ASI_Disabled(self):
    """UOD.applicationSpecificInputs: control flag is disabled......................................"""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = dict()
    self.uod.applicationSpecificInputs()
    self.assertFalse(self.uod.enable)

  def test_ASI_AllVariables(self):
    """UOD.applicationSpecificInputs: checks if all variables have been properly set after this call"""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = dict(JobReport=self.jr_mock, FileReport=self.fr_mock, PRODUCTION_ID=43321,
                                      JOB_ID=12345,
                                      outputList=[{'outputFile': 'myFile_gen.stdhep'},
                                                    {'outputFile': 'myFile_dst.slcio'},
                                                    {'outputFile': 'myFile_sim.slcio'},
                                                    {'outputFile': 'myFile_rec.slcio'},
                                                    {'outputFile': 'myFile_unk.slcio'}
                                                   ],
                                      ProductionOutputData="/my/long/path/GEN/myFile_gen_12345_001.stdhep;/my/long/path/DST/myFile_dst_12345_001.slcio"
                                    )
    os.environ['JOBID'] = "12345"
    self.uod.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertIsInstance(self.uod.outputDataFileMask, list)
    self.assertTrue(self.uod.outputMode)
    self.assertTrue(self.uod.outputList)
    self.assertTrue(self.uod.productionID)
    self.assertTrue(self.uod.prodOutputLFNs)
    self.assertTrue(self.uod.experiment)

  def test_ASI_NoVariables(self):
    """UOD.applicationSpecificInputs: checks that no variables have been set after this call........"""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = dict()
    os.environ['JOBID'] = "12345"
    self.uod.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertFalse(self.uod.jobReport or self.uod.productionID)

  def test_ASI_OutputListCorrect(self):
    """UOD.applicationSpecificInputs: check outputfile list is treated properly....................."""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = {'outputList': [{'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
                                                 'outputFile': 'h_nunu_gen.stdhep',
                                                 'outputDataSE': 'CERN-SRM'},
                                                {'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/SIM',
                                                 'outputFile': 'h_nunu_sim.slcio',
                                                 'outputDataSE': 'CERN-SRM'},
                                                {'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC',
                                                 'outputFile': 'h_nunu_rec.slcio',
                                                 'outputDataSE': 'CERN-SRM'},
                                                {'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/DST',
                                                 'outputFile': 'h_nunu_dst.slcio',
                                                 'outputDataSE': 'CERN-SRM'}],
                                 'ProductionOutputData':
                                 '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep;/ilc/prod/clic/1.4tev/h_nunu/SID/SIM/00004192/001/h_nunu_sim_4192_1002.slcio;/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/002/h_nunu_rec_4193_2766.slcio;/ilc/prod/clic/1.4tev/h_nunu/SID/DST/00004193/002/h_nunu_dst_4193_2766.slcio'}
    os.environ['JOBID'] = "12345"
    self.uod.applicationSpecificInputs()
    del os.environ['JOBID']
    self.assertEqual(len(self.uod.outputList), 4)

  def test_GOL_Reco(self):
    """outputList properly formated for reconstruction jobs........................................."""
    gLogger.setLevel("ERROR")
    wf_real = {'TaskID': '2766',
               'TotalSteps': '4',
               'JobName': '00004193_00002766',
               'Priority': '1',
               'SoftwarePackages': 'overlayinput.1;lcsim.CLIC_CDR;slicpandora.CLIC_CDR_photon_fix',
               'DebugLFNs': '',
               'Status': 'Created',
               'JobReport': self.jr_mock,
               'BannedSites': 'LCG.Bristol.uk;LCG.RAL-LCG2.uk',
               'LogLevel': 'verbose',
               'StdOutput': 'std.out',
               'JobType': 'MCReconstruction_Overlay',
               'SystemConfig': 'x86_64-slc5-gcc43-opt',
               'TransformationID': '4193',
               'JOB_ID': '00002766',
               'productionVersion': '$Id: ',
               'StdError': 'std.err',
               'LogTargetPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/LOG/00004193_2766.tar',
               'IS_PROD': 'True',
               'Request': self.rc_mock,
               'ParametricInputSandbox': '',
               'emailAddress': 'stephane.poss@cern.ch',
               'JobGroup': '00004193',
               'NbOfEvts': 200,
               'Origin': 'DIRAC',
               'outputList': [{'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC',
                               'outputFile': 'h_nunu_rec.slcio',
                               'outputDataSE': 'CERN-SRM'},
                              {'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/DST',
                               'outputFile': 'h_nunu_dst.slcio',
                               'outputDataSE': 'CERN-SRM'}],
               'Energy': 1400.0,
               'ProductionOutputData': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/002/h_nunu_rec_4193_2766.slcio;/ilc/prod/clic/1.4tev/h_nunu/SID/DST/00004193/002/h_nunu_dst_4193_2766.slcio',
               'Site': 'ANY',
               'OwnerGroup': 'ilc_prod',
               'PRODUCTION_ID': '00004193',
               'Owner': 'sailer',
               'MaxCPUTime': '300000',
               'LogFilePath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/LOG/002',
               'InputData': '/ilc/prod/clic/1.4tev/h_nunu/SID/SIM/00004192/000/h_nunu_sim_4192_655.slcio'}
    self.uod = UploadOutputData()
    self.uod.workflow_commons = wf_real
    self.uod.outputList = wf_real['outputList']
    proddata = self.uod.workflow_commons['ProductionOutputData'].split(";")
    olist = {}
    for obj in self.uod.outputList:
      self.uod.getTreatedOutputlistNew(proddata, olist, obj)

    filesFound = [f in olist for f in ('h_nunu_dst_4193_2766', 'h_nunu_rec_4193_2766')]
    self.assertTrue(all(filesFound))

  def test_GOL_RecoNew(self):
    """outputList properly formated for reconstruction jobs........................................."""
    gLogger.setLevel("ERROR")
    wf_real = {'TaskID': '2766',
               'TotalSteps': '4',
               'JobName': '00004193_00002766',
               'Priority': '1',
               'SoftwarePackages': 'overlayinput.1;lcsim.CLIC_CDR;slicpandora.CLIC_CDR_photon_fix',
               'DebugLFNs': '',
               'Status': 'Created',
               'JobReport': self.jr_mock,
               'BannedSites': 'LCG.Bristol.uk;LCG.RAL-LCG2.uk',
               'LogLevel': 'verbose',
               'StdOutput': 'std.out',
               'JobType': 'MCReconstruction_Overlay',
               'SystemConfig': 'x86_64-slc5-gcc43-opt',
               'TransformationID': '4193',
               'JOB_ID': '00002766',
               'productionVersion': '$Id: ',
               'StdError': 'std.err',
               'LogTargetPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/LOG/00004193_2766.tar',
               'IS_PROD': 'True',
               'Request': self.rc_mock,
               'ParametricInputSandbox': '',
               'emailAddress': 'stephane.poss@cern.ch',
               'JobGroup': '00004193',
               'NbOfEvts': 200,
               'Origin': 'DIRAC',
               'outputList': [{'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC',
                               'outputFile': 'h_nunu_rec.slcio',
                               'outputDataSE': 'CERN-SRM'},
                              {'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/SID/DST',
                               'outputFile': 'h_nunu_dst.slcio',
                               'outputDataSE': 'CERN-SRM'}],
               'Energy': 1400.0,
               'ProductionOutputData': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/002/h_nunu_rec_4193_2766.slcio;/ilc/prod/clic/1.4tev/h_nunu/SID/DST/00004193/002/h_nunu_dst_4193_2766.slcio',
               'Site': 'ANY',
               'OwnerGroup': 'ilc_prod',
               'PRODUCTION_ID': '00004193',
               'Owner': 'sailer',
               'MaxCPUTime': '300000',
               'LogFilePath': '/ilc/prod/clic/1.4tev/h_nunu/SID/REC/00004193/LOG/002',
               'InputData': '/ilc/prod/clic/1.4tev/h_nunu/SID/SIM/00004192/000/h_nunu_sim_4192_655.slcio'}
    self.uod = UploadOutputData()
    self.uod.workflow_commons = wf_real
    self.uod.outputList = wf_real['outputList']
    proddata = self.uod.workflow_commons['ProductionOutputData'].split(";")
    olist = {}
    for obj in self.uod.outputList:
      self.uod.getTreatedOutputlistNew(proddata, olist, obj)

    filesFound = [f in olist for f in ('h_nunu_dst_4193_2766', 'h_nunu_rec_4193_2766')]
    self.assertTrue(all(filesFound))

  def test_GOL_gen(self):
    """outputList properly formated for reconstruction jobs........................................."""
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.workflow_commons = {
        'outputList': [
            {
                'outputPath': '/ilc/prod/clic/1.4tev/h_nunu/gen',
                'outputFile': 'h_nunu_gen.stdhep',
                'outputDataSE': 'CERN-SRM'}],
        'ProductionOutputData': '/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0000.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0001.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0002.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0003.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0004.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0005.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0006.stdhep;/ilc/prod/clic/1.4tev/h_nunu/GEN/00004191/000/h_nunu_gen_4191_0007.stdhep;'}

    self.uod.outputList = self.uod.workflow_commons['outputList']
    proddata = self.uod.workflow_commons['ProductionOutputData'].split(";")
    olist = {}
    for obj in self.uod.outputList:
      self.uod.getTreatedOutputlistNew(proddata, olist, obj)

    filesFound = [f in olist for f in ('h_nunu_gen_4191_0000',
                                       'h_nunu_gen_4191_0001',
                                       'h_nunu_gen_4191_0002',
                                       'h_nunu_gen_4191_0003',
                                       'h_nunu_gen_4191_0004',
                                       'h_nunu_gen_4191_0005',
                                       'h_nunu_gen_4191_0006',
                                       'h_nunu_gen_4191_0007')]
    self.assertTrue(all(filesFound))

  @patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations', new=Mock())
  @patch("%s.ModuleBase.RequestValidator" % MODULE_NAME, new=Mock())
  def test_EXE_cleanUpRequests(self):
    """execute: test when Requests are being cleaned up ............................................"""
    # we want that the upload output data fails to upload and do registration,
    # so we need to create a registration request, by failing at the right
    # place in the FailoverTransfer, which means we need to somehow control
    # the failovertransfer that is being called by the UploadOutputData...
    #
    gLogger.setLevel("ERROR")
    self.uod = UploadOutputData()
    self.uod.prodOutputLFNs = ['/ilc/user/s/sailer/test3.stdhep']
    self.uod.workflow_commons = dict(Enable=True)
    candidateFiles = {'test3.stdhep': {'lfn': '/ilc/user/s/sailer/test3.stdhep', 'workflowSE': 'CERN-DIP-4'}}
    self.uod.getCandidateFiles = Mock(return_value=S_OK())
    res = self.uod.getFileMetadata(candidateFiles)
    self.uod.getFileMetadata = Mock(return_value=res)
    self.uod.getDestinationSEList = Mock(return_value=S_OK(["CERN-DIP-4"]))
    self.uod.enable = True
    self.uod.jobID = 12345
    from DIRAC.DataManagementSystem.Client.FailoverTransfer import FailoverTransfer
    FailoverTransfer.transferAndRegisterFile = Mock(return_value=S_ERROR("IT ACTUALLY WORKS!!!!!!1eleven!!"))
    _resUodExe = self.uod.execute()
    res = self.uod.generateFailoverFile()

#############################################################################
# UserJobFinalization.py
#############################################################################


@patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
@patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
class TestUserJobFinalization(ModulesTestCase):
  """test UserJobFinalization."""

  @patch("%s.ModuleBase.getProxyInfoAsString" % MODULE_NAME, new=Mock(return_value=S_OK()))
  @patch("DIRAC.Core.Security.ProxyInfo.getProxyInfoAsString", new=Mock(return_value=S_OK()))
  def setUp(self):
    super(TestUserJobFinalization, self).setUp()
    with patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.Operations', Mock()):
      self.ujf = UserJobFinalization()

  def test_UJF_execute_isLastStep(self):
    """UJF.execute: is last step...................................................................."""
    self.ujf.step_commons['STEP_NUMBER'] = 2
    self.ujf.workflow_commons['TotalSteps'] = 2
    resLS = self.ujf.isLastStep()
    self.assertTrue(resLS['OK'])

  def test_UJF_execute_isLastStep_not(self):
    """UJF.execute: is Not the last step............................................................"""
    self.ujf.step_commons['STEP_NUMBER'] = 1
    self.ujf.workflow_commons['TotalSteps'] = 2
    resLS = self.ujf.isLastStep()
    self.assertFalse(resLS['OK'])

  def test_UFJ_getOutputList(self):
    """UJF.execute: getOutputList..................................................................."""
    gLogger.setLevel("ERROR")
    self.ujf.userOutputSE = "CERN-SRM"
    self.ujf.userOutputData = ['gen.stdhep',
                               'sim.slcio',
                               'rec.slcio',
                               'dst.slcio']

    outputList = self.ujf.getOutputList()
    self.log.debug(outputList)

  def test_UJF_TRFF(self):
    """UJF.execute: transferAndRegisterFailoverFile................................................."""
    gLogger.setLevel("ERROR")
    ft_mock = Mock()
    ft_mock.transferAndRegisterFileFailover.return_value = S_OK()
    candidateFiles = {'test3.stdhep': {'lfn': '/ilc/user/s/sailer/test3.stdhep', 'workflowSE': 'CERN-SRM'}}
    resMetadata = self.ujf.getFileMetadata(candidateFiles)
    filesToFailover = resMetadata['Value']
    filesToFailover['test3.stdhep']['resolvedSE'] = ['CERN-SRM', 'KEK-SRM', 'RAL-SRM']
    filesUploaded = []
    self.ujf.failoverSEs = ['CERN-SRM', 'RAL-SRM']
    res = self.ujf.transferRegisterAndFailoverFiles(ft_mock, filesToFailover, filesUploaded)
    self.log.debug(res)
    self.log.debug(filesUploaded)
    self.log.debug(res)
    self.assertFalse(res['Value']['cleanUp'] and filesUploaded)

  def test_UJF_TRFF_Failed(self):
    """UJF.execute: transferAndRegisterFailoverFile, no more SEs...................................."""
    gLogger.setLevel("ERROR")
    ft_mock = Mock()
    ft_mock.transferAndRegisterFileFailover.return_value = S_OK()
    filesToFailover = {'test.txt': {'lfn': '/ilc/user/s/sailer/test/test.txt',
                                     'localpath': './test.txt',
                                     'resolvedSE': ['CERN-SRM', 'KEK-SRM', 'RAL-SRM'],
                                     'workflowSE': ['CERN-SRM'],
                                     'path': 'SLCIO',
                                     'guid': 'A331AE88-AD87-AF39-97E1-44257D8200C8'}}
    filesUploaded = []
    self.ujf.failoverSEs = ['CERN-SRM']
    res = self.ujf.transferRegisterAndFailoverFiles(ft_mock, filesToFailover, filesUploaded)
    self.log.debug(res)
    self.log.debug(filesUploaded)
    self.log.debug(res)
    self.assertTrue(res['Value']['cleanUp'] and not filesUploaded)


if __name__ == '__main__':
  unittest.main(verbosity=2)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run SLIC.

:since:  Apr 7, 2010

:author: Stephane Poss
"""


from __future__ import absolute_import
import os

from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Workflow.Utilities.CompactMixin import CompactMixin
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getEnvironmentScript
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import prepareMacFile, getNewLDLibs
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import resolveIFpaths, getProdFilename
from ILCDIRAC.Core.Utilities.FindSteeringFileDir import getSteeringFileDirName

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class SLICAnalysis(CompactMixin, ModuleBase):
  """Specific Module to run a SLIC job."""

  def __init__(self):
    super(SLICAnalysis, self).__init__()
    self.enable = True
    self.STEP_NUMBER = ''
    self.result = S_ERROR()
    self.applicationName = 'SLIC'
    self.startFrom = 0
    self.RandomSeed = 0
    self.detectorModel = ''
    self.eventstring = ['BeginEvent']

  def applicationSpecificInputs(self):
    """Resolve all input variables for the module here.

    :return: S_OK()
    """

    if self.WorkflowStartFrom:
      self.startFrom = self.WorkflowStartFrom

    # Move below to ModuleBase as common to Mokka
    if "IS_PROD" in self.workflow_commons:
      self.RandomSeed = int(
          str(
              int(
                  self.workflow_commons["PRODUCTION_ID"]))
          + str(
              int(
                  self.workflow_commons["JOB_ID"])))
    elif self.jobID:
      self.RandomSeed = self.jobID

    if 'IS_PROD' in self.workflow_commons and self.workflow_commons["IS_PROD"]:
      self.OutputFile = getProdFilename(self.OutputFile,
                                        int(self.workflow_commons["PRODUCTION_ID"]),
                                        int(self.workflow_commons["JOB_ID"]))

    if not len(self.InputFile) and len(self.InputData):
      for files in self.InputData:
        if files.lower().find(".stdhep") > -1 or files.lower().find(".hepevt") > -1:
          self.InputFile.append(files)
          break

    return S_OK('Parameters resolved')

  def runIt(self):
    """Called by JobAgent.

    Execute the following:
      - get the environment variables that should have been set during installation
      - download the detector model, using CS query to fetch the address
      - prepare the mac file using :any:`prepareMacFile`
      - run SLIC on this mac File and catch the exit status

    :return: S_OK(), S_ERROR()
    """
    self.result = S_OK()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result['Message'])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('SLIC should not proceed as previous step did not end properly')

    res = getEnvironmentScript(self.platform, self.applicationName, self.applicationVersion, self.getEnvScript)
    if not res['OK']:
      LOG.error("Could not obtain the environment script: ", res["Message"])
      return res
    env_script_path = res["Value"]

    retMod = self.getDetectorModel()
    if not retMod:
      return retMod

    slicmac = 'slicmac.mac'
    if len(self.InputFile):
      res = resolveIFpaths(self.InputFile)
      if not res['OK']:
        LOG.error("Generator file not found")
        return res
      self.InputFile = res['Value']

    if len(self.SteeringFile) > 0:
      self.SteeringFile = os.path.basename(self.SteeringFile)
      if not os.path.exists(self.SteeringFile):
        res = getSteeringFileDirName(self.platform, self.applicationName, self.applicationVersion)
        if not res['OK']:
          LOG.error("Could not find where the steering files are")
          return res
        steeringfiledirname = res['Value']
        if os.path.exists(os.path.join(steeringfiledirname, self.SteeringFile)):
          self.SteeringFile = os.path.join(steeringfiledirname, self.SteeringFile)
      if not os.path.exists(self.SteeringFile):
        LOG.error("Missing steering file")
        return S_ERROR("Could not find mac file")
    # Same as for mokka: using ParticleGun does not imply InputFile
    if not len(self.InputFile):
      self.InputFile = ['']
    macok = prepareMacFile(self.SteeringFile, slicmac, self.InputFile[0],
                           self.NumberOfEvents, self.startFrom, self.detectorModel,
                           self.RandomSeed, self.OutputFile, self.debug)
    if not macok['OK']:
      LOG.error('Failed to create SLIC mac file')
      return S_ERROR('Error when creating SLIC mac file')

    scriptName = 'SLIC_%s_Run_%s.sh' % (self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = open(scriptName, 'w')
    script.write('#!/bin/bash \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')
    script.write("source %s\n" % (env_script_path))

    preloadDir = "%s/preload" % os.getcwd()
    if os.path.exists(preloadDir):
      preadloadLibs = set(os.listdir(preloadDir))
      preloadString = ":".join(lib for lib in preadloadLibs if ".so" in lib)
      script.write('declare -x LD_LIBRARY_PATH=%s:${LD_LIBRARY_PATH}\n' % preloadDir)
      script.write('declare -x LD_PRELOAD=%s\n' % preloadString)

    script.write('echo =========\n')
    script.write('env | sort >> localEnv.log\n')
    script.write('echo SLIC:\n')
    script.write("which slic\n")
    script.write('echo =========\n')
    comm = 'slic -P $PARTICLE_TBL -m %s %s\n' % (slicmac, self.extraCLIarguments)
    LOG.info("Command:", comm)
    script.write(comm)
    script.write('declare -x appstatus=$?\n')
    script.write('exit $appstatus\n')
    script.close()
    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    os.chmod(scriptName, 0o755)
    comm = 'sh -c "./%s"' % scriptName
    self.setApplicationStatus('SLIC %s step %s' % (self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    self.result = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    #self.result = {'OK':True,'Value':(0,'Disabled Execution','')}
    resultTuple = self.result['Value']
    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('%s failed terribly, you are doomed!' % (self.applicationName))
      if not self.ignoreapperrors:
        return S_ERROR('%s did not produce the expected log' % (self.applicationName))
    status = resultTuple[0]
    # stdOutput = resultTuple[1]
    # stdError = resultTuple[2]
    LOG.info("Status after the application execution is %s" % str(status))

    return self.finalStatusReport(status)

  def getEnvScript(self, sysconfig, appname, appversion):
    """This is called in case CVMFS is not there."""
    if 'SLIC_DIR' not in os.environ:
      LOG.error('SLIC_DIR not found, probably the software installation failed')
      return S_ERROR('SLIC_DIR not found, probably the software installation failed')
    if 'SLIC_VERSION' not in os.environ:
      LOG.error('SLIC_VERSION not found, probably the software installation failed')
      return S_ERROR('SLIC_VERSION not found, probably the software installation failed')
    if 'LCDD_VERSION' not in os.environ:
      LOG.error('LCDD_VERSION not found, probably the software installation failed')
      return S_ERROR('LCDD_VERSION not found, probably the software installation failed')
    # if 'XERCES_VERSION' not in os.environ:
    #  LOG.error('XERCES_VERSION not found, probably the software installation failed')
    #  return S_ERROR('XERCES_VERSION not found, probably the software installation failed')

    # Need to fetch the new LD_LIBRARY_PATH
    new_ld_lib_path = getNewLDLibs(sysconfig, appname, appversion)
    #res = getSoftwareFolder(sysconfig, appname, appversion)
    # if not res['OK']:
    #  LOG.error('Directory %s was not found in either the local area or shared area' % (slicDir))
    #  return res
    mySoftwareRoot = os.environ['SLIC_DIR']
    env_name = "SLICEnv.sh"
    script = open(env_name, "w")
    script.write("#/bin/sh\n")
    script.write("######################\n")
    script.write("## Env script for SLIC\n")
    script.write("######################\n")
    if 'XERCES_VERSION' in os.environ:
      script.write('declare -x XERCES_LIB_DIR=%s/packages/xerces/%s/lib\n' % (mySoftwareRoot,
                                                                              os.environ['XERCES_VERSION']))
      if new_ld_lib_path:
        script.write('declare -x LD_LIBRARY_PATH=$XERCES_LIB_DIR:%s\n' % new_ld_lib_path)
      else:
        script.write('declare -x LD_LIBRARY_PATH=$XERCES_LIB_DIR\n')

    script.write('declare -x GEANT4_DATA_ROOT=%s/packages/geant4/data\n' % mySoftwareRoot)
    script.write('declare -x G4LEVELGAMMADATA=$(ls -d $GEANT4_DATA_ROOT/PhotonEvaporation*)\n')
    script.write('declare -x G4RADIOACTIVEDATA=$(ls -d $GEANT4_DATA_ROOT/RadioactiveDecay*)\n')
    script.write('declare -x G4LEDATA=$(ls -d $GEANT4_DATA_ROOT/G4EMLOW*)\n')
    script.write('declare -x G4NEUTRONHPDATA=$(ls -d $GEANT4_DATA_ROOT/G4NDL*)\n')
    script.write('declare -x GDML_SCHEMA_DIR=%s/packages/lcdd/%s\n' % (mySoftwareRoot, os.environ['LCDD_VERSION']))
    script.write('declare -x PARTICLE_TBL=%s/packages/slic/%s/data/particle.tbl\n' % (mySoftwareRoot,
                                                                                      os.environ['SLIC_VERSION']))
    script.write('declare -x MALLOC_CHECK_=0\n')
    if os.path.exists("%s/lib" % (mySoftwareRoot)):
      script.write('declare -x LD_LIBRARY_PATH=%s/lib:$LD_LIBRARY_PATH\n' % (mySoftwareRoot))
    script.write('declare -x PATH=%s/packages/slic/%s/bin/Linux-g++/:$PATH\n' % (mySoftwareRoot,
                                                                                 os.environ['SLIC_VERSION']))
    script.close()
    os.chmod(env_name, 0o755)
    return S_OK(os.path.abspath(env_name))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""SID DBD specific production job utility.

:author: S. Poss
:since: Jul 01, 2012
"""

from __future__ import absolute_import
import string
from decimal import Decimal

from DIRAC.Core.Workflow.Module import ModuleDefinition
from DIRAC.Core.Workflow.Step import StepDefinition
from DIRAC import S_OK, S_ERROR

from ILCDIRAC.Interfaces.API.NewInterface.ProductionJob import ProductionJob
from ILCDIRAC.Interfaces.Utilities import JobHelpers
import six


__RCSID__ = "$Id$"


class SIDProductionJob(ProductionJob):
  """Production Job for SID."""

  def __init__(self):
    super(SIDProductionJob, self).__init__()
    self.basepath = self.ops.getValue('/Production/ILC_SID/BasePath', '/ilc/prod/ilc/sid/')
    self.polarization = ""
    self.machineparams = ''
    self.detector = ''

  def setInputDataQuery(self, metadata):
    """Define the input data query needed, also get from the data the meta info requested to build the path."""
    retMetaKey = self._checkMetaKeys(list(metadata.keys()))
    if not retMetaKey['OK']:
      return retMetaKey

    retDirs = self._checkFindDirectories(metadata)
    if not retDirs['OK']:
      return retDirs
    dirs = list(retDirs['Value'].values())
    for mdir in dirs:
      res = self.fc.getDirectoryUserMetadata(mdir)
      if not res['OK']:
        return self._reportError("Error looking up the catalog for directory metadata")
      compatmeta = res['Value']
      compatmeta.update(metadata)

    if 'EvtType' in compatmeta:
      self.evttype = JobHelpers.getValue(compatmeta['EvtType'], str, six.string_types)
    else:
      return self._reportError("EvtType is not in the metadata, it has to be!")

    if 'NumberOfEvents' in compatmeta:
      self.nbevts = JobHelpers.getValue(compatmeta['NumberOfEvents'], int, None)

    if 'Energy' in compatmeta:
      self.energycat = JobHelpers.getValue(compatmeta['Energy'], str, six.string_types)

    if 'Polarisation' in compatmeta:
      self.polarization = JobHelpers.getValue(compatmeta["Polarisation"], str, six.string_types)

    if 'MachineParams' in compatmeta:
      self.machineparams = JobHelpers.getValue(compatmeta["MachineParams"], str, six.string_types)

    gendata = False
    if 'Datatype' in compatmeta:
      self.datatype = JobHelpers.getValue(compatmeta['Datatype'], str, six.string_types)
      if self.datatype == 'GEN':
        gendata = True

    if 'DetectorModel' in compatmeta and not gendata:
      self.detector = JobHelpers.getValue(compatmeta["DetectorModel"], str, six.string_types)

    self.basename = self.evttype + "_" + self.polarization

    self.energy = Decimal(self.energycat)

    self.inputBKSelection = metadata
    self.prodparameters['nbevts'] = self.nbevts
    self.prodparameters["FCInputQuery"] = self.inputBKSelection

    self.inputdataquery = True
    return S_OK()

  def _addRealFinalization(self):
    """See :any:`ProductionJob` for definition."""
    importLine = 'from ILCDIRAC.Workflow.Modules.<MODULE> import <MODULE>'
    dataUpload = ModuleDefinition('UploadOutputData')
    dataUpload.setDescription('Uploads the output data')
    self._addParameter(dataUpload, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'UploadOutputData')
    dataUpload.setBody(body)

    failoverRequest = ModuleDefinition('FailoverRequest')
    failoverRequest.setDescription('Sends any failover requests')
    self._addParameter(failoverRequest, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'FailoverRequest')
    failoverRequest.setBody(body)

    registerdata = ModuleDefinition('SIDRegisterOutputData')
    registerdata.setDescription('Module to add in the metadata catalog the relevant info about the files')
    self._addParameter(registerdata, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'SIDRegisterOutputData')
    registerdata.setBody(body)

    logUpload = ModuleDefinition('UploadLogFile')
    logUpload.setDescription('Uploads the output log files')
    self._addParameter(logUpload, 'enable', 'bool', False, 'EnableFlag')
    body = importLine.replace('<MODULE>', 'UploadLogFile')
    logUpload.setBody(body)

    finalization = StepDefinition('Job_Finalization')
    finalization.addModule(dataUpload)
    up = finalization.createModuleInstance('UploadOutputData', 'dataUpload')
    up.setValue("enable", self.finalsdict['uploadData'])

    finalization.addModule(registerdata)
    ro = finalization.createModuleInstance('SIDRegisterOutputData', 'SIDRegisterOutputData')
    ro.setValue("enable", self.finalsdict['registerData'])

    finalization.addModule(logUpload)
    ul = finalization.createModuleInstance('UploadLogFile', 'logUpload')
    ul.setValue("enable", self.finalsdict['uploadLog'])

    finalization.addModule(failoverRequest)
    fr = finalization.createModuleInstance('FailoverRequest', 'failoverRequest')
    fr.setValue("enable", self.finalsdict['sendFailover'])

    self.workflow.addStep(finalization)
    self.workflow.createStepInstance('Job_Finalization', 'finalization')

    return S_OK()

  def _jobSpecificParams(self, application):
    """For production additional checks are needed: ask the user."""

    if self.created:
      return S_ERROR("The production was created, you cannot add new applications to the job.")

    if not application.logFile:
      logf = application.appname + "_" + application.version + "_@{STEP_ID}.log"
      res = application.setLogFile(logf)
      if not res['OK']:
        return res

      # in fact a bit more tricky as the log files have the prodID and jobID in them

    # Retrieve from the application the essential info to build the prod info.
    if not self.nbevts:
      self.nbevts = application.numberOfEvents
      if not self.nbevts:
        return S_ERROR("Number of events to process is not defined.")
    elif not application.numberOfEvents:
      res = application.setNumberOfEvents(self.jobFileGroupSize * self.nbevts)
      if not res['OK']:
        return res

    if application.numberOfEvents > 0 and self.jobFileGroupSize * self.nbevts > application.numberOfEvents:
      self.nbevts = application.numberOfEvents

    if self.prodparameters["SWPackages"]:
      curpackage = "%s.%s" % (application.appname, application.version)
      if not self.prodparameters["SWPackages"].count(curpackage):
        self.prodparameters["SWPackages"] += ";%s" % (curpackage)
    else:
      self.prodparameters["SWPackages"] = "%s.%s" % (application.appname, application.version)

    if not self.energy:
      if application.energy:
        self.energy = Decimal(str(application.energy))
      else:
        return S_ERROR("Could not find the energy defined, it is needed for the production definition.")
    elif not application.energy:
      res = application.setEnergy(float(self.energy))
      if not res['OK']:
        return res
    if self.energy:
      self._setParameter("Energy", "float", float(self.energy), "Energy used")
      self.prodparameters["Energy"] = float(self.energy)

    if not self.evttype:
      if hasattr(application, 'eventType'):
        self.evttype = application.eventType
      else:
        return S_ERROR("Event type not found nor specified, it's mandatory for the production paths.")

    if not application.accountInProduction:
      # needed for the stupid overlay
      res = self._updateProdParameters(application)
      if not res['OK']:
        return res
      self.checked = True

      return S_OK()

    if not self.outputStorage:
      return S_ERROR("You need to specify the Output storage element")

    res = application.setOutputSE(self.outputStorage)
    if not res['OK']:
      return res

    if not self.detector:
      if hasattr(application, "detectorModel"):
        self.detector = application.detectorModel
        if not self.detector:
          return S_ERROR("Application does not know which model to use, so the production does not either.")
      # else:
      #  return S_ERROR("Application does not know which model to use, so the production does not either.")

    # Below modify according to SID conventions
    energypath = "%s_%s/" % (self.energy, self.polarization)  # 1000_p80m20
    # self.finalMetaDict[self.basepath+energypath] = {'Energy' : str(self.energy),
    #                                                "Polarisation" : self.polarization,
    #                                                "MachineParams" : self.machineparams}

    if not self.basename:
      self.basename = self.evttype

    if not self.evttype[-1] == '/':
      evttypemeta = self.evttype
      self.evttype += '/'
    else:
      evttypemeta = self.evttype.rstrip("/")

    if self.detector:
      if not self.detector[-1] == "/":
        detectormeta = self.detector
        self.detector += "/"
      else:
        detectormeta = self.detector.rstrip("/")

    path = self.basepath
    # Need to resolve file names and paths
    if hasattr(application, "setOutputRecFile") and not application.willBeCut:
      path = self.basepath + energypath + self.evttype + self.detector + "REC/"
      self.finalMetaDict[self.basepath + energypath + self.evttype] = {"EvtType": evttypemeta}
      self.finalMetaDict[self.basepath + energypath + self.evttype + self.detector] = {"DetectorModel": detectormeta}
      self.finalMetaDict[self.basepath + energypath + self.evttype + self.detector + "REC"] = {'Datatype': "REC"}
      fname = self.basename + "_rec.slcio"
      application.setOutputRecFile(fname, path)
      self.finalpaths.append(path)
      path = self.basepath + energypath + self.evttype + self.detector + "DST/"
      self.finalMetaDict[self.basepath + energypath + self.evttype + self.detector] = {"DetectorModel": detectormeta}
      self.finalMetaDict[self.basepath + energypath + self.evttype + self.detector + "DST"] = {'Datatype': "DST"}
      fname = self.basename + "_dst.slcio"
      application.setOutputDstFile(fname, path)
      self.finalpaths.append(path)
    elif hasattr(application, "outputFile") and hasattr(application, 'datatype') and (not application.outputFile) and (not application.willBeCut):
      path = self.basepath + energypath + self.evttype
      self.finalMetaDict[path] = {"EvtType": evttypemeta}
      if hasattr(application, "detectorModel"):
        if application.detectorModel:
          path += application.detectorModel
          self.finalMetaDict[path] = {"DetectorModel": application.detectorModel}
          path += '/'
        elif self.detector:
          path += self.detector
          self.finalMetaDict[path] = {"DetectorModel": self.detector}
          path += '/'
      if (not application.datatype) and self.datatype:
        application.datatype = self.datatype
      path += application.datatype

      self.finalMetaDict[path] = {"Datatype": application.datatype}
      self.log.info("Will store the files under", "%s" % path)
      self.finalpaths.append(path)

      extension = 'stdhep'
      if application.datatype in ['SIM', 'REC']:
        extension = 'slcio'
      fname = self.basename + "_%s" % (application.datatype.lower()) + "." + extension
      application.setOutputFile(fname, path)

    self.basepath = path

    res = self._updateProdParameters(application)
    if not res['OK']:
      return res

    self.checked = True

    return S_OK()



#!/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""submit jobs to all sites and checks the worker nodes for functionality."""
from __future__ import absolute_import
__RCSID__ = "$Id$"
from DIRAC.Core.Base import Script
from DIRAC import S_OK

from DIRAC import gLogger

LOG = gLogger.getSubLogger(__name__)


class _Params(object):
  def __init__(self):
    self.site = None
    self.ce = None

  def setSite(self, opt):
    self.site = opt
    return S_OK()

  def setCE(self, opt):
    self.ce = opt
    return S_OK()

  def registerSwitches(self):
    Script.registerSwitch("S:", "Site=", "Site to probe", self.setSite)
    Script.registerSwitch("C:", "CE=", "Computing Element to probe", self.setCE)
    Script.setUsageMessage("%s --Site LCG.CERN.ch" % Script.scriptName)


def testAndProbeSites():
  """submits jobs to test sites."""
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine()

  from DIRAC import exit as dexit

  from ILCDIRAC.Interfaces.API.NewInterface.UserJob import UserJob
  from ILCDIRAC.Interfaces.API.NewInterface.Applications import CheckWNs
  from ILCDIRAC.Interfaces.API.DiracILC import DiracILC

  from DIRAC.ConfigurationSystem.Client.Helpers.Resources import getQueues

  res = getQueues(siteList=clip.site, ceList=clip.ce)
  if not res['OK']:
    LOG.error("Failed getting the queues", res['Message'])
    dexit(1)

  sitedict = res['Value']
  CEs = []

  for ces in sitedict.values():
    CEs.extend(list(ces.keys()))

  LOG.notice("Found %s CEs to look at." % len(CEs))

  d = DiracILC(True, "SiteProbe.rep")

  for CE in CEs:
    j = UserJob()
    j.setDestinationCE(CE)
    c = CheckWNs()
    res = j.append(c)
    if not res['OK']:
      LOG.error(res['Message'])
      continue
    j.setOutputSandbox("*.log")
    j.setCPUTime(30000)
    j.dontPromptMe()
    res = j.submit(d)
    if not res['OK']:
      LOG.error("Failed to submit job, aborting")
      dexit(1)

  dexit(0)


if __name__ == "__main__":
  testAndProbeSites()
